@@ -42,7 +42,6 @@
 #include "htmlengine-save.h"
 #include "htmlpainter.h"
 #include "htmlplainpainter.h"
-#include "htmlprinter.h"
 #include "htmlsearch.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
@@ -59,13 +58,13 @@ static HTMLClueClass *parent_class = NULL;
 static gchar *        get_item_marker_str                   (HTMLClueFlow *flow, gboolean ascii_only);
 static guint          get_post_padding                      (HTMLClueFlow *flow,
 							     guint pad);
-static gint            get_similar_depth                     (HTMLClueFlow *self,
+static int            get_similar_depth                     (HTMLClueFlow *self,
 							     HTMLClueFlow *neighbor);
 
 static void
 copy_levels (GByteArray *dst, GByteArray *src)
 {
-	gint i;
+	int i;
 
 	g_byte_array_set_size (dst, src->len);
 
@@ -267,7 +266,7 @@ op_helper (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left,
 	if (!from && prev_flow_in_cluevs (self)) {
 		(*len) ++;
 		/* if (cut)
-		   e->cursor->position--; */
+		   e->cursor->position --; */
 	}
 	if (cut)
 		html_clue_remove_text_slaves (HTML_CLUE (self));
@@ -391,9 +390,9 @@ merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **
 			cf2->item_type = cf1->item_type;
 
 			update_item_number (self, e);
-			cf1->item_number--;
+			cf1->item_number --;
 			update_item_number (with, e);
-			cf1->item_number++;
+			cf1->item_number ++;
 
 			if (cf2_next_relative)
 				update_item_number (cf2_next_relative, e);
@@ -441,7 +440,7 @@ is_header (HTMLClueFlow *flow)
 static gboolean
 need_blockquote_padding  (HTMLClueFlow *flow, HTMLClueFlow *prev)
 {
-	gint i = get_similar_depth (flow, prev);
+	int i = get_similar_depth (flow, prev);
 
 	/*
 	 * If the levels don't match up the the current flow
@@ -463,7 +462,7 @@ need_blockquote_padding  (HTMLClueFlow *flow, HTMLClueFlow *prev)
 	while (i > 0) {
 		HTMLListType type;
 
-		type = prev->levels->data[prev->levels->len - i];
+		type = prev->levels->data [prev->levels->len - i];
 
 		if (is_blockquote (type)) {
 			return TRUE;
@@ -499,20 +498,20 @@ get_pre_padding (HTMLClueFlow *flow, guint pad)
 				return pad;
 			else
 				return 0;
-		}
+  		}
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && prev->style != HTML_CLUEFLOW_STYLE_PRE
-		    && !is_header (prev))
+		    && ! is_header (prev))
 			return pad;
 
-		if (is_header (flow) && !is_header (prev))
+		if (is_header (flow) && ! is_header (prev))
 			return pad;
 
 		return 0;
 	}
 
-	if (!is_header (flow) && flow->levels->len == 0)
+	if (! is_header (flow) && flow->levels->len == 0)
 		return 0;
 
 	return pad;
@@ -542,7 +541,7 @@ get_post_padding (HTMLClueFlow *flow,
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && next->style != HTML_CLUEFLOW_STYLE_PRE
-		    && !is_header (next))
+		    && ! is_header (next))
 			return pad;
 
 		if (is_header (flow))
@@ -551,7 +550,7 @@ get_post_padding (HTMLClueFlow *flow,
 		return 0;
 	}
 
-	if (!is_header (flow) && flow->levels->len == 0)
+	if (! is_header (flow) && flow->levels->len == 0)
 		return 0;
 
 	return pad;
@@ -590,7 +589,7 @@ get_level_indent (HTMLClueFlow *flow,
 	guint indent = 0;
 	gint i = 0;
 
-	if (flow->levels->len > 0 || !is_item (flow)) {
+	if (flow->levels->len > 0 || ! is_item (flow)) {
 		guint cite_width, indent_width;
 
 		cite_width = html_painter_get_block_cite_width (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL, dir)
@@ -666,7 +665,7 @@ calc_min_width (HTMLObject *o,
 	gint w = 0;
 	gboolean add;
 
-	add = HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE && !HTML_IS_PRINTER (painter);
+	add = HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE;
 
 	cur = HTML_CLUE (o)->head;
 	while (cur) {
@@ -688,13 +687,13 @@ calc_min_width (HTMLObject *o,
 }
 
 static gint
-pref_left_margin (HTMLPainter *p, HTMLObject *o, gint indent)
+pref_left_margin (HTMLPainter *p, HTMLObject *o, int indent)
 {
-	gint margin = html_object_get_left_margin (o->parent, p, o->y, TRUE);
+	int margin = html_object_get_left_margin (o->parent, p, o->y, TRUE);
 
 	if (html_object_get_direction (o) == HTML_DIRECTION_RTL) {
-		if ((HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE || HTML_IS_PRINTER (p)) && HTML_IS_PLAIN_PAINTER (p))
-			return MAX (margin, o->width - (gint)(72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
+		if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && HTML_IS_PLAIN_PAINTER(p))
+			return MAX (margin, o->width - (int)(72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
 									html_painter_get_e_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL)))));
 	} else {
 		if (indent > margin)
@@ -705,12 +704,12 @@ pref_left_margin (HTMLPainter *p, HTMLObject *o, gint indent)
 }
 
 static gint
-pref_right_margin (HTMLPainter *p, HTMLObject *o, gint indent)
+pref_right_margin (HTMLPainter *p, HTMLObject *o, int indent)
 {
 	gint margin = html_object_get_right_margin (o->parent, p, o->y, TRUE);
 
 	if (html_object_get_direction (o) != HTML_DIRECTION_RTL) {
-		if ((HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE || HTML_IS_PRINTER (p)) && HTML_IS_PLAIN_PAINTER (p))
+		if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && HTML_IS_PLAIN_PAINTER(p))
 			return MIN (margin, 72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
 						       html_painter_get_e_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL))));
 	} else {
@@ -747,9 +746,9 @@ calc_margins (HTMLObject *o, HTMLPainter *painter, gint indent, gint *lmargin, g
 }
 
 static inline gint
-width_left (HTMLObject *o, gint x, gint rmargin, gboolean printing)
+width_left (HTMLObject *o, gint x, gint rmargin)
 {
-	return (!printing && HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE) ? G_MAXINT : rmargin - x;
+	return HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE ? G_MAXINT : rmargin - x;
 }
 
 static gint
@@ -889,6 +888,7 @@ update_line_positions (HTMLObject *clue, HTMLObject *begin, HTMLObject *end, gin
 	}
 }
 
+
 static HTMLObject *
 layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	     GList **changed_objs, gboolean *leaf_children_changed_size,
@@ -949,13 +949,13 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 						  indent, &o->y, lmargin, rmargin);
 
 			/* is there enough space for this object? */
-			if ((HTML_IS_PRINTER (painter) || HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE) && o->y != old_y && *rmargin - x < nb_width)
+			if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && o->y != old_y && *rmargin - x < nb_width)
 				break;
 			need_update_height = TRUE;
 		}
 
 		cur->y = o->ascent + a;
-		fit = html_object_fit_line (cur, painter, first, first, FALSE, width_left (o, x, *rmargin, HTML_IS_PRINTER (painter)));
+		fit = html_object_fit_line (cur, painter, first, first, FALSE, width_left (o, x, *rmargin));
 		first = FALSE;
 		if (fit == HTML_FIT_NONE)
 			break;
@@ -987,7 +987,7 @@ layout_aligned (HTMLObject *o, HTMLPainter *painter, HTMLObject *cur,
 		GList **changed_objs, gboolean *leaf_children_changed_size,
 		gint *lmargin, gint *rmargin, gint indent, gboolean *changed)
 {
-	if (!html_clue_appended (HTML_CLUE (o->parent), HTML_CLUE (cur))) {
+	if (! html_clue_appended (HTML_CLUE (o->parent), HTML_CLUE (cur))) {
 		html_object_calc_size (cur, painter, changed_objs);
 
 		if (HTML_CLUE (cur)->halign == HTML_HALIGN_LEFT)
@@ -1129,8 +1129,7 @@ static gchar *
 get_roman_value (gint value, gboolean lower)
 {
 	GString *str;
-	const gchar *base = "IVXLCDM";
-	gchar *rv;
+	gchar *rv, *base = "IVXLCDM";
 	gint b, r, add = lower ? 'a' - 'A' : 0;
 
 	if (value > 3999)
@@ -1143,19 +1142,19 @@ get_roman_value (gint value, gboolean lower)
 		if (r != 0) {
 			if (r < 4) {
 				for (; r; r--)
-					g_string_prepend_c (str, base[b] + add);
+					g_string_prepend_c (str, base [b] + add);
 			} else if (r == 4) {
-				g_string_prepend_c (str, base[b + 1] + add);
-				g_string_prepend_c (str, base[b] + add);
+				g_string_prepend_c (str, base [b + 1] + add);
+				g_string_prepend_c (str, base [b] + add);
 			} else if (r == 5) {
-				g_string_prepend_c (str, base[b + 1] + add);
+				g_string_prepend_c (str, base [b + 1] + add);
 			} else if (r < 9) {
 				for (; r > 5; r--)
-					g_string_prepend_c (str, base[b] + add);
-				g_string_prepend_c (str, base[b + 1] + add);
+					g_string_prepend_c (str, base [b] + add);
+				g_string_prepend_c (str, base [b + 1] + add);
 			} else if (r == 9) {
-				g_string_prepend_c (str, base[b + 2] + add);
-				g_string_prepend_c (str, base[b] + add);
+				g_string_prepend_c (str, base [b + 2] + add);
+				g_string_prepend_c (str, base [b] + add);
 			}
 		}
 	}
@@ -1172,11 +1171,11 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 	HTMLListType type = flow->item_type;
 
 	if (type == HTML_LIST_TYPE_BLOCKQUOTE && flow->levels->len > 0) {
-		gint i;
+		int i;
 
-		for (i = flow->levels->len - 1; i >= 0; i--) {
-			if (flow->levels->data[i] != HTML_LIST_TYPE_BLOCKQUOTE) {
-				type = flow->levels->data[i];
+		for (i = flow->levels->len - 1; i >= 0; i --) {
+			if (flow->levels->data [i] != HTML_LIST_TYPE_BLOCKQUOTE) {
+				type = flow->levels->data [i];
 				break;
 			}
 		}
@@ -1191,12 +1190,6 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 	case HTML_LIST_TYPE_ORDERED_LOWER_ROMAN:
 	case HTML_LIST_TYPE_ORDERED_UPPER_ROMAN:
 		return get_roman_value (flow->item_number, flow->item_type == HTML_LIST_TYPE_ORDERED_LOWER_ROMAN);
-	case HTML_LIST_TYPE_CIRCLE:
-		return g_strdup ("\342\227\213 "); /* U+25CB WHITE CIRCLE */
-	case HTML_LIST_TYPE_DISC:
-		return g_strdup ("\342\227\217 "); /* U+25CF BLACK CIRCLE */
-	case HTML_LIST_TYPE_SQUARE:
-		return g_strdup ("\342\226\240 "); /* U+25AA BLACK SQUARE */
 	case HTML_LIST_TYPE_UNORDERED:
 		if (ascii_only)
 			return g_strdup ("* ");
@@ -1210,7 +1203,7 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 }
 
 static void
-draw_cite_line (HTMLObject *cur, HTMLPainter *p, const gchar *cite_str, gint offset, gint x, gint y)
+draw_cite_line (HTMLObject *cur, HTMLPainter *p, char *cite_str, gint offset, gint x, gint y)
 {
 	gint cy, w, a, d;
 
@@ -1240,9 +1233,9 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 {
 	HTMLClueFlow *flow;
 	GdkRectangle paint, area, clip;
-	gint i;
-	gint indent = 0;
-	gint last_indent = 0;
+	int i;
+	int indent = 0;
+	int last_indent = 0;
 	gint pixel_size = html_painter_get_pixel_size (painter);
 	gboolean is_plain = HTML_IS_PLAIN_PAINTER (painter);
 	HTMLDirection dir = html_object_get_direction (self);
@@ -1283,16 +1276,17 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 							paint.width, paint.height);
 			} else {
 				HTMLObject *cur = HTML_CLUE (self)->head;
-				gint x_pos, baseline = 0;
-				const gchar *cite_str = dir == HTML_DIRECTION_RTL ? HTML_BLOCK_CITE_RTL : HTML_BLOCK_CITE_LTR;
+				int x_pos, baseline = 0;
+				char *cite_str = dir == HTML_DIRECTION_RTL ? HTML_BLOCK_CITE_RTL : HTML_BLOCK_CITE_LTR;
 
 				while (cur) {
 					if (cur->y != 0) {
 						baseline = cur->y;
 						break;
-					}
-					cur = cur->next;
-				}
+  					}
+  					cur = cur->next;
+  				}
+
 
 				/* draw "> " quote characters in the plain case */
 				html_painter_set_font_style (painter, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED);
@@ -1490,7 +1484,7 @@ write_indent (HTMLEngineSaveState *state, gint level)
 	while (level > 0) {
 		if (!html_engine_save_output_string (state, "    "))
 			return FALSE;
-		level--;
+		level --;
 	}
 
 	return TRUE;
@@ -1525,7 +1519,8 @@ get_list_start_tag (HTMLClueFlow *self)
 	return NULL;
 }
 
-static const gchar *
+
+static gchar *
 get_start_tag (HTMLClueFlow *self)
 {
 	switch (self->style) {
@@ -1553,7 +1548,7 @@ get_start_tag (HTMLClueFlow *self)
 	}
 }
 
-static const gchar *
+static const char *
 get_start_indent_item (HTMLListType type)
 {
 	switch (type) {
@@ -1578,17 +1573,11 @@ get_start_indent_item (HTMLListType type)
 		return "BLOCKQUOTE TYPE=CITE";
 	case HTML_LIST_TYPE_BLOCKQUOTE:
 		return "BLOCKQUOTE";
-	case HTML_LIST_TYPE_CIRCLE:
-		return "OL TYPE=CIRCLE";
-	case HTML_LIST_TYPE_DISC:
-		return "OL TYPE=DISC";
-	case HTML_LIST_TYPE_SQUARE:
-		return "OL TYPE=SQUARE";
 	}
 	return "";
 }
 
-static const gchar *
+static const char *
 get_end_indent_item (HTMLListType type)
 {
 	switch (type) {
@@ -1608,11 +1597,11 @@ get_end_indent_item (HTMLListType type)
 	return "";
 }
 
-static gint
+static int
 get_similar_depth (HTMLClueFlow *self, HTMLClueFlow *neighbor)
 {
-	gint i;
-	gint max_depth;
+	int i;
+	int max_depth;
 
 	if (neighbor == NULL)
 		return 0;
@@ -1628,7 +1617,7 @@ get_similar_depth (HTMLClueFlow *self, HTMLClueFlow *neighbor)
 }
 
 static gboolean
-save_indent_string (HTMLClueFlow *self, HTMLEngineSaveState *state, const gchar *format, ...)
+save_indent_string (HTMLClueFlow *self, HTMLEngineSaveState *state, const char *format, ...)
 {
 	va_list args;
 	gboolean retval;
@@ -1644,10 +1633,10 @@ save_indent_string (HTMLClueFlow *self, HTMLEngineSaveState *state, const gchar
 	return retval;
 }
 
-static const gchar *
+static char *
 get_p_str (HTMLClueFlow *self, HTMLEngineSaveState *state)
 {
-	const gchar *p_str = NULL;
+	char *p_str = NULL;
 
 	if (self->dir != html_object_get_direction (state->engine->clue)) {
 		switch (self->dir) {
@@ -1672,8 +1661,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	HTMLClueFlow *next = NULL;
 	HTMLClueFlow *prev = NULL;
 	HTMLHAlignType halign;
-	const gchar *br_str = "<BR>\n";
-	gboolean has_div_tag = FALSE;
+	char *br_str = "<BR>\n";
 
 	if (HTML_IS_CLUEFLOW (HTML_OBJECT (self)->next))
 		next = HTML_CLUEFLOW (HTML_OBJECT (self)->next);
@@ -1696,16 +1684,16 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	}
 
 	if (!prev) {
-		const gchar *p_str = get_p_str (self, state);
+		char *p_str = get_p_str (self, state);
 
 		if (p_str) {
-			if (!html_engine_save_output_string (state, "%s", p_str))
+			if (! html_engine_save_output_string (state, p_str))
 				return FALSE;
 		}
 	}
 
 	if (is_item (self)) {
-		gchar *li = get_list_start_tag (self);
+		char *li = get_list_start_tag (self);
 
 		if (li && !save_indent_string (self, state, "<%s>", li)) {
 			g_free (li);
@@ -1715,7 +1703,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 		if (!save_indent_string (self, state, ""))
 			return FALSE;
 	} else {
-		const gchar *start = get_start_tag (self);
+		char *start = get_start_tag (self);
 
 		if (start) {
 			if (!save_indent_string (self, state, "<%s>\n", start))
@@ -1728,9 +1716,8 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 
 	halign = HTML_CLUE (self)->halign;
 	/* Alignment tag.  */
-	has_div_tag = halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT;
-	if (has_div_tag) {
-		if (!html_engine_save_output_string
+	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
+		if (! html_engine_save_output_string
 		    (state, "<DIV ALIGN=%s>",
 		     html_engine_save_get_paragraph_align (html_alignment_to_paragraph (halign))))
 			return FALSE;
@@ -1740,35 +1727,35 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 		return FALSE;
 
 	/* Paragraph's content.  */
-	if (!HTML_OBJECT_CLASS (&html_clue_class)->save (HTML_OBJECT (self), state))
+	if (! HTML_OBJECT_CLASS (&html_clue_class)->save (HTML_OBJECT (self), state))
 		return FALSE;
 
 	/* Close alignment tag.  */
-	if (has_div_tag) {
-		if (!html_engine_save_output_string (state, "</DIV>"))
+	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
+		if (! html_engine_save_output_string (state, "</DIV>"))
 			return FALSE;
 	}
 
 	if (is_item (self)) {
-		if (!has_div_tag && next && is_levels_equal (self, next) && !is_item (next) && !html_clueflow_contains_table (self)) {
-			if (!html_engine_save_output_string (state, "%s", br_str))
+		if (next && is_levels_equal (self, next) && !is_item (next) && !html_clueflow_contains_table (self)) {
+			if (!html_engine_save_output_string (state, br_str))
 				return FALSE;
 		} else if (!html_engine_save_output_string (state, "\n"))
 			return FALSE;
 	} else if (is_levels_equal (self, next) && self->style == next->style) {
-		if (!has_div_tag && self->style != HTML_CLUEFLOW_STYLE_PRE && !html_clueflow_contains_table (self)) {
-			if (!html_engine_save_output_string (state, "%s", br_str))
+		if (self->style != HTML_CLUEFLOW_STYLE_PRE && !html_clueflow_contains_table (self)) {
+			if (!html_engine_save_output_string (state, br_str))
 				return FALSE;
 		} else {
 			if (!html_engine_save_output_string (state, "\n"))
 				return FALSE;
 		}
 	} else {
-		const gchar *end = get_start_tag (self);
+		char *end = get_start_tag (self);
 
 		if (self->style != HTML_CLUEFLOW_STYLE_PRE) {
-			if (!has_div_tag && ((!html_clueflow_contains_table (self) && !end && next && self->style == next->style) || html_clueflow_is_empty (self))) {
-				if (!html_engine_save_output_string (state, "%s", br_str))
+			if ((!html_clueflow_contains_table (self) && !end && next && self->style == next->style) || html_clueflow_is_empty (self)) {
+				if (!html_engine_save_output_string (state, br_str))
 					return FALSE;
 			} else {
 				if (!html_engine_save_output_string (state, "\n"))
@@ -1790,10 +1777,10 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 			HTMLObject *head = HTML_CLUE (HTML_OBJECT (self)->parent)->head;
 
 			if (head && HTML_IS_CLUEFLOW (head)) {
-				const gchar *head_p_str = get_p_str (HTML_CLUEFLOW (head), state);
+				char *head_p_str = get_p_str (HTML_CLUEFLOW (head), state);
 
 				if (head_p_str) {
-					if (!html_engine_save_output_string (state, "</P>\n"))
+					if (! html_engine_save_output_string (state, "</P>\n"))
 						return FALSE;
 				}
 			}
@@ -1810,8 +1797,8 @@ save (HTMLObject *s,
 	HTMLClueFlow *self = HTML_CLUEFLOW (s);
 	HTMLClueFlow *next = NULL;
 	HTMLClueFlow *prev = NULL;
-	gint d;
-	gint i;
+	int d;
+	int i;
 
 	if (HTML_IS_CLUEFLOW (HTML_OBJECT (self)->next))
 		next = HTML_CLUEFLOW (HTML_OBJECT (self)->next);
@@ -1819,9 +1806,9 @@ save (HTMLObject *s,
 	if (HTML_IS_CLUEFLOW (HTML_OBJECT (self)->prev))
 		prev = HTML_CLUEFLOW (HTML_OBJECT (self)->prev);
 
-	i = get_similar_depth (self, prev);
+	d = i = get_similar_depth (self, prev);
 	while (i < self->levels->len) {
-		const gchar *stag = get_start_indent_item (self->levels->data[i]);
+		const char *stag = get_start_indent_item (self->levels->data[i]);
 
 		if (!write_indent (state, i)
 		    || !html_engine_save_output_string (state, "<%s>\n", stag))
@@ -1836,7 +1823,7 @@ save (HTMLObject *s,
 	i = self->levels->len - 1;
 	d = get_similar_depth (self, next);
 	while (i >= d) {
-		const gchar *stag = get_end_indent_item (self->levels->data[i]);
+		const char *stag = get_end_indent_item (self->levels->data[i]);
 
 		if (!write_indent (state, i)
 		    || !html_engine_save_output_string (state, "</%s>\n", stag))
@@ -1851,14 +1838,14 @@ save (HTMLObject *s,
 static void
 write_item_marker (GString *pad_string, HTMLClueFlow *flow)
 {
-	gchar *marker;
+	char *marker;
 
 	marker = get_item_marker_str (flow, TRUE);
 
 	if (marker) {
 		gint marker_len = strlen (marker);
 		gint len = pad_string->len - 1;
-		gchar *str = pad_string->str;
+		char *str = pad_string->str;
 
 		while (len > 0) {
 			if ((str[len - 1] != ' ') || (pad_string->len - len >= marker_len))
@@ -1935,7 +1922,7 @@ append_selection_string (HTMLObject *self,
  * zero-width or combined characters.)
  */
 static gint
-utf8_width (const gchar *str, gint len)
+utf8_width (const char *str, gint len)
 {
 	gunichar c;
 	gint width = 0;
@@ -1953,7 +1940,7 @@ utf8_width (const gchar *str, gint len)
  * not exceeding the given width.
  */
 static gint
-utf8_length_in_width (const gchar *str, gint len, gint width)
+utf8_length_in_width (const char *str, gint len, gint width)
 {
 	gunichar c;
 	gint l = 0;
@@ -1995,17 +1982,17 @@ save_plain (HTMLObject *self,
 							      buffer_state,
 							      max_width)) {
 		guchar *s;
-		gint offset;
+		int offset;
 
 		if (get_pre_padding (flow, calc_padding (state->engine->painter)) > 0) {
-			plain_padding (flow, out, FALSE);
+		        plain_padding (flow, out, FALSE);
 			g_string_append (out, "\n");
 		}
 
 		s = html_engine_save_buffer_peek_text (buffer_state);
 
 		if (*s == 0) {
-			plain_padding (flow, out, TRUE);
+		        plain_padding (flow, out, TRUE);
 			g_string_append (out, "\n");
 		} else {
 			PangoAttrList *attrs = pango_attr_list_new ();
@@ -2020,23 +2007,23 @@ save_plain (HTMLObject *self,
 			lattrs = g_new (PangoLogAttr, slen + 1);
 			n_items = g_list_length (items_list);
 			items = g_new (PangoItem *, n_items);
-			for (i = 0, cur = items_list; i < n_items; i++, cur = cur->next)
-				items[i] = (PangoItem *) cur->data;
+			for (i = 0, cur = items_list; i < n_items; i ++, cur = cur->next)
+				items [i] = (PangoItem *) cur->data;
 
 			offset = 0;
-			for (i = 0; i < n_items; i++) {
+			for (i = 0; i < n_items; i ++) {
 				PangoItem tmp_item;
-				gint start_offset;
+				int start_offset;
 
 				start_offset = offset;
-				offset += items[i]->num_chars;
-				tmp_item = *items[i];
+				offset += items [i]->num_chars;
+				tmp_item = *items [i];
 				while (i < n_items - 1) {
-					if (tmp_item.analysis.lang_engine == items[i + 1]->analysis.lang_engine) {
-						tmp_item.length += items[i + 1]->length;
-						tmp_item.num_chars += items[i + 1]->num_chars;
-						offset += items[i + 1]->num_chars;
-						i++;
+					if (tmp_item.analysis.lang_engine == items [i + 1]->analysis.lang_engine) {
+						tmp_item.length += items [i + 1]->length;
+						tmp_item.num_chars += items [i + 1]->num_chars;
+						offset += items [i + 1]->num_chars;
+						i ++;
 					} else
 						break;
 				}
@@ -2047,8 +2034,8 @@ save_plain (HTMLObject *self,
 			html_text_remove_unwanted_line_breaks ((gchar *) s, slen, lattrs);
 
 			g_list_free (items_list);
-			for (i = 0; i < n_items; i++)
-				pango_item_free (items[i]);
+			for (i = 0; i < n_items; i ++)
+				pango_item_free (items [i]);
 			g_free (items);
 			pango_attr_list_unref (attrs);
 
@@ -2069,12 +2056,12 @@ save_plain (HTMLObject *self,
 						wmax = clen + utf8_length_in_width ((gchar *) s, len, max_width);
 						wl = wmax;
 
-						if (lattrs[wl].is_white) {
+						if (lattrs [wl].is_white) {
 
-							while (lattrs[wl].is_white && wl < slen)
-								wl++;
+							while (lattrs [wl].is_white && wl < slen)
+								wl ++;
 
-							if (wl < slen && html_text_is_line_break (lattrs[wl]))
+							if (wl < slen && html_text_is_line_break (lattrs [wl]))
 								look_backward = FALSE;
 							else
 								wl = wmax;
@@ -2082,16 +2069,16 @@ save_plain (HTMLObject *self,
 
 						if (look_backward) {
 							while (wl > 0) {
-								if (html_text_is_line_break (lattrs[wl]))
+								if (html_text_is_line_break (lattrs [wl]))
 									break;
-								wl--;
+								wl --;
 							}
 						}
 
-						if (wl > clen && wl < slen && html_text_is_line_break (lattrs[wl])) {
+						if (wl > clen && wl < slen && html_text_is_line_break (lattrs [wl])) {
 							wi = MIN (wl, wmax);
-							while (wi > clen && lattrs[wi - 1].is_white)
-								wi--;
+							while (wi > clen && lattrs [wi - 1].is_white)
+								wi --;
 							len = wi - clen;
 							width = utf8_width ((gchar *) s, len);
 							skip = wl - wi;
@@ -2135,7 +2122,7 @@ save_plain (HTMLObject *self,
 
 				if (*s == '\n') {
 					s++;
-					clen++;
+					clen ++;
 				}
 
 				g_string_append_c (out, '\n');
@@ -2264,12 +2251,12 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 
 	if (text_bytes > 0) {
 		par = (guchar *) g_new (gchar, text_bytes + 1);
-		par[text_bytes] = 0;
+		par [text_bytes] = 0;
 
 		pp = (info->forward) ? par : par + text_bytes;
 
 		/* now fill par with text */
-		head = (info->forward) ? *beg : end;
+		head = cur = (info->forward) ? *beg : end;
 		cur = *beg;
 		while (cur) {
 			if (html_object_is_text (cur)) {
@@ -2292,19 +2279,19 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 			if (info->start_pos > 0)
 				index = ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos + ((info->forward) ? 0 : -1))) - par;
 			else
-				index = info->forward ? ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos) - par) : -1;
+				index = ((guchar *)g_utf8_offset_to_pointer ((gchar *) par, info->start_pos)) - par;
 		} else {
-			index = (info->forward) ? 0 : text_bytes;
+			index = (info->forward) ? 0 : text_bytes - 1;
 		}
 
-		/* make shorter text instead */
-		if (!info->forward && index + info->text_bytes < text_bytes) {
-			guchar * tmp = (guchar *)(par + index + info->text_bytes);
+		/* FIXME make shorter text instead */
+		if (!info->forward) {
+			guchar* tmp = (guchar *)g_utf8_next_char (par + index);
 			*tmp = '\0';
 		}
 
 		if ((info->forward && index < text_bytes)
-		    || (!info->forward && index >= 0)) {
+		    || (!info->forward && index > 0)) {
 			if (info->reb) {
 				/* regex search */
 				gint rv;
@@ -2356,36 +2343,36 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 			} else {
 				/* substring search - simple one - could be improved
 				   go thru par and look for info->text */
-				while ((info->forward && par[index])
-				       || (!info->forward && index >= 0)) {
+				while (par [index]) {
 					gunichar unicode_info, unicode_par;
 
-					unicode_info = g_utf8_get_char (info->text + eq_bytes);
+					unicode_info = g_utf8_get_char (((info->forward)
+									? (info->text + eq_bytes)
+									: g_utf8_prev_char (info->text + info->text_bytes - eq_bytes)));
 					unicode_par = g_utf8_get_char ((gchar *) par + index);
 					if (!info->case_sensitive) {
                                              unicode_info = g_unichar_toupper (unicode_info);
-					     unicode_par  = g_unichar_toupper (unicode_par);
+				 	     unicode_par  = g_unichar_toupper (unicode_par);
 					}
 
-					if (unicode_info == unicode_par) {
+               				if (unicode_info == unicode_par) {
 						eq_bytes += (guchar *)g_utf8_next_char (par + index) - par - index;
 
 						if (eq_bytes == info->text_bytes) {
 							/* The above par + index is always at the beginning of the last character matched */
-							index = (guchar *) g_utf8_next_char ((gchar *) par + index) - par - eq_bytes;
+							if (info->forward)
+								index = (guchar *) g_utf8_next_char ((gchar *) par + index) - par - eq_bytes;
 							search_set_info (head, info, par, index, info->text_bytes);
 							retval=TRUE;
 							break;
 						}
 					} else {
-						index -= eq_bytes;
+						index += (info->forward) ? -eq_bytes : eq_bytes;
 						eq_bytes = 0;
 					}
-					index += (info->forward || unicode_info == unicode_par)
+					index += (info->forward)
 						? (((guchar *) g_utf8_next_char (par + index)) - par - index)
-						: (index
-							? (((guchar *) g_utf8_prev_char ((gchar *) par + index)) - par - index)
-							: -1);
+						: (((guchar *) g_utf8_prev_char ((gchar *) par + index)) - par - index);
 				}
 			}
 		}
@@ -2555,7 +2542,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 void
 html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 		    HTMLClueFlowStyle style, GByteArray *levels, HTMLListType item_type, gint item_number,
-		    HTMLClearType clear)
+ 		    HTMLClearType clear)
 {
 	HTMLObject *object;
 	HTMLClue *clue;
@@ -2842,7 +2829,7 @@ html_clueflow_set_indentation (HTMLClueFlow *flow,
 			       guint8 *indentation_levels)
 {
 	HTMLObject *next_relative;
-	gint i;
+	int i;
 	g_return_if_fail (flow != NULL);
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
@@ -2869,7 +2856,7 @@ html_clueflow_get_indentation (HTMLClueFlow *flow)
 {
 	g_return_val_if_fail (flow != NULL, 0);
 
-	/* FIXME levels */
+	// FIXME levels
 	return flow->levels->len;
 }
 
@@ -2904,7 +2891,7 @@ html_clueflow_get_properties (HTMLClueFlow *flow,
 	if (style_return != NULL)
 		*style_return = flow->style;
 	if (indentation_return != NULL)
-		/* FIXME levels */
+		// FIXME levels
 		*indentation_return = flow->levels->len;
 	if (alignment_return != NULL)
 		*alignment_return = HTML_CLUE (flow)->halign;
@@ -2945,7 +2932,7 @@ get_text (HTMLClue *clue, HTMLInterval *i)
 
 	bytes        = get_text_bytes (clue, i);
 	ct           = text = g_malloc (bytes + 1);
-	text[bytes] = 0;
+	text [bytes] = 0;
 
 	obj = html_interval_get_head (i, HTML_OBJECT (clue));
 	while (obj) {
