@@ -38,20 +38,21 @@
 #include "htmltokenizer.h"
 #include "htmlembedded.h"
 
+
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
 
 static void
-iframe_set_base (GtkHTML *html, const gchar *url, gpointer data)
+iframe_set_base (GtkHTML *html, const char *url, gpointer data)
 {
-	gchar *new_url = gtk_html_get_url_base_relative (html, url);
+	char *new_url = gtk_html_get_url_base_relative (html, url);
 
 	gtk_html_set_base (html, new_url);
 	g_free (new_url);
 }
 
 static void
-iframe_url_requested (GtkHTML *html, const gchar *url, GtkHTMLStream *handle, gpointer data)
+iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
 {
 	HTMLIFrame *iframe = HTML_IFRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
@@ -95,7 +96,7 @@ iframe_set_gdk_painter (HTMLIFrame *iframe, HTMLPainter *painter)
 
 HTMLObject *
 html_iframe_new (GtkWidget *parent,
-		 gchar *src,
+		 char *src,
 		 gint width,
 		 gint height,
 		 gboolean border)
@@ -169,7 +170,7 @@ draw (HTMLObject *o,
 	HTMLEngine   *e       = GTK_HTML (iframe->html)->engine;
 	GdkRectangle paint;
 
-	if (G_OBJECT_TYPE (e->painter) == HTML_TYPE_PRINTER) {
+	if (GTK_OBJECT_TYPE (e->painter) == HTML_TYPE_PRINTER) {
 		gint pixel_size = html_painter_get_pixel_size (e->painter);
 
 		if (!html_object_intersect (o, &paint, x, y, width, height))
@@ -215,7 +216,7 @@ static gint
 check_page_split (HTMLObject *self, HTMLPainter *p, gint y)
 {
 	HTMLEngine *e = GTK_HTML (HTML_IFRAME (self)->html)->engine;
-	gint y1, y2, pixel_size = html_painter_get_pixel_size (p);
+	int y1, y2, pixel_size = html_painter_get_pixel_size (p);
 
 	y1 = self->y - self->ascent + pixel_size*html_engine_get_top_border (e);
 	y2 = self->y + self->descent + pixel_size*html_engine_get_bottom_border (e);
@@ -272,29 +273,6 @@ op_copy (HTMLObject *self, HTMLObject *parent, HTMLEngine *e, GList *from, GList
 	return dup;
 }
 
-static HTMLAnchor *
-find_anchor (HTMLObject *self, const gchar *name, gint *x, gint *y)
-{
-	HTMLIFrame *iframe;
-	HTMLAnchor *anchor;
-
-	g_return_val_if_fail (HTML_IS_IFRAME (self), NULL);
-
-	iframe = HTML_IFRAME (self);
-
-	if (!iframe || !iframe->html || !GTK_IS_HTML (iframe->html) || !GTK_HTML (iframe->html)->engine || !GTK_HTML (iframe->html)->engine->clue)
-		return NULL;
-
-	anchor = html_object_find_anchor (GTK_HTML (iframe->html)->engine->clue, name, x, y);
-
-	if (anchor) {
-		*x += self->x;
-		*y += self->y - self->ascent;
-	}
-
-	return anchor;
-}
-
 void
 html_iframe_set_margin_width (HTMLIFrame *iframe, gint margin_width)
 {
@@ -492,7 +470,7 @@ save (HTMLObject *s,
 		}
 		html_engine_save_buffer_free (buffer, TRUE);
 	} else {
-		if (!html_engine_save_delims_and_vals (state, "<IFRAME SRC=\"", iframe->url, "\"", NULL))
+		if (!html_engine_save_output_string (state, "<IFRAME SRC=\"%s\"", iframe->url))
 			 return FALSE;
 
 		 if (iframe->width >= 0)
@@ -568,7 +546,7 @@ void
 html_iframe_init (HTMLIFrame *iframe,
 		  HTMLIFrameClass *klass,
 		  GtkWidget *parent,
-		  gchar *src,
+		  char *src,
 		  gint width,
 		  gint height,
 		  gboolean border)
@@ -610,11 +588,7 @@ html_iframe_init (HTMLIFrame *iframe,
 	new_tokenizer = NULL;
 
 	gtk_html_set_default_content_type (new_html,
-					   gtk_html_get_default_content_type(parent_html));
-
-	gtk_html_set_default_engine (new_html,
-					   gtk_html_get_default_engine(parent_html));
-
+					   parent_html->priv->content_type);
 	iframe->html = new_widget;
 	iframe->url = g_strdup (src);
 	iframe->width = width;
@@ -648,15 +622,20 @@ html_iframe_init (HTMLIFrame *iframe,
 	 * on_url and link_clicked are emitted from the toplevel widget not
 	 * proxied like url_requested is.
 	 */
-	g_signal_connect (new_html, "on_url", G_CALLBACK (iframe_on_url), iframe);
-	g_signal_connect (new_html, "link_clicked", G_CALLBACK (iframe_link_clicked), iframe);
+	gtk_signal_connect (GTK_OBJECT (new_html), "on_url",
+			    GTK_SIGNAL_FUNC (iframe_on_url),
+			    (gpointer)iframe);
+	gtk_signal_connect (GTK_OBJECT (new_html), "link_clicked",
+			    GTK_SIGNAL_FUNC (iframe_link_clicked),
+			    (gpointer)iframe);
 #endif
 	g_signal_connect (new_html, "size_changed", G_CALLBACK (iframe_size_changed), iframe);
 	g_signal_connect (new_html, "set_base", G_CALLBACK (iframe_set_base), iframe);
 	g_signal_connect (new_html, "object_requested", G_CALLBACK (iframe_object_requested), iframe);
 
 	/*
-	  g_signal_connect (html, "button_press_event", G_CALLBACK (iframe_button_press_event), iframe);
+	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
+	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
 	*/
 
 	gtk_widget_set_size_request (scrolled_window, width, height);
@@ -674,10 +653,14 @@ html_iframe_init (HTMLIFrame *iframe,
 				     parent_html->engine->settings->color_set);
 	html_painter_set_focus (new_html->engine->painter, parent_html->engine->have_focus);
 	/*
-	g_signal_connect (html, "title_changed", G_CALLBACK (title_changed_cb), (gpointer)app);
-	g_signal_connect (html, "button_press_event", G_CALLBACK (on_button_press_event), popup_menu);
-	g_signal_connect (html, "redirect", G_CALLBACK (on_redirect), NULL);
-	g_signal_connect (html, "object_requested", G_CALLBACK (object_requested_cmd), NULL);
+	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
+			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
+	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
+			    GTK_SIGNAL_FUNC (on_button_press_event), popup_menu);
+	gtk_signal_connect (GTK_OBJECT (html), "redirect",
+			    GTK_SIGNAL_FUNC (on_redirect), NULL);
+	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
+			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
 	*/
 }
 
@@ -690,7 +673,7 @@ html_iframe_type_init (void)
 void
 html_iframe_class_init (HTMLIFrameClass *klass,
 			HTMLType type,
-			guint size)
+		        guint size)
 {
 	HTMLEmbeddedClass *embedded_class;
 	HTMLObjectClass  *object_class;
@@ -723,7 +706,6 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->check_point             = check_point;
 	object_class->is_container            = is_container;
 	object_class->append_selection_string = append_selection_string;
-	object_class->find_anchor             = find_anchor;
 
 	embedded_class->reparent = reparent;
 }
