@@ -24,7 +24,13 @@
 */
 
 #include <config.h>
+#include <glib.h>
 #include <string.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 
 #include "gtkhtml.h"
 #include "gtkhtml-properties.h"
@@ -63,7 +69,7 @@ struct _HTMLImageFactory {
 static HTMLImageClass html_image_class;
 static HTMLObjectClass *parent_class = NULL;
 
-static HTMLImagePointer   *html_image_pointer_new               (const gchar *filename, HTMLImageFactory *factory);
+static HTMLImagePointer   *html_image_pointer_new               (const char *filename, HTMLImageFactory *factory);
 static void                html_image_pointer_ref               (HTMLImagePointer *ip);
 static void                html_image_pointer_unref             (HTMLImagePointer *ip);
 static gboolean            html_image_pointer_timeout           (HTMLImagePointer *ip);
@@ -92,7 +98,7 @@ html_image_get_actual_width (HTMLImage *image, HTMLPainter *painter)
 		width = gdk_pixbuf_animation_get_width (anim) * pixel_size;
 
 		if (image->specified_height > 0 || image->percent_height) {
-			gdouble scale;
+			double scale;
 
 			scale =  ((double) html_image_get_actual_height (image, painter))
 				/ (gdk_pixbuf_animation_get_height (anim) * pixel_size);
@@ -125,7 +131,7 @@ html_image_get_actual_height (HTMLImage *image, HTMLPainter *painter)
 		height = gdk_pixbuf_animation_get_height (anim) * pixel_size;
 
 		if (image->specified_width > 0 || image->percent_width) {
-			gdouble scale;
+			double scale;
 
 			scale = ((double) html_image_get_actual_width (image, painter))
 				/ (gdk_pixbuf_animation_get_width (anim) * pixel_size);
@@ -149,9 +155,8 @@ destroy (HTMLObject *o)
 {
 	HTMLImage *image = HTML_IMAGE (o);
 
-	if (image->image_ptr->factory)
-		html_image_factory_unregister (image->image_ptr->factory,
-					       image->image_ptr, HTML_IMAGE (image));
+	html_image_factory_unregister (image->image_ptr->factory,
+				       image->image_ptr, HTML_IMAGE (image));
 
 	g_free (image->url);
 	g_free (image->target);
@@ -213,7 +218,7 @@ image_update_url (HTMLImage *image, gint x, gint y)
 {
 	HTMLMap *map;
 	HTMLObject *o = HTML_OBJECT (image);
-	gchar *url = NULL;
+	char *url = NULL;
 
 	/*
 	 * FIXME this is a huge hack waiting until we implement events for now we write
@@ -599,14 +604,14 @@ save (HTMLObject *self,
 	if (image->url && *image->url) {
 		url  = g_strconcat (image->url, image->target ? "#" : "", image->target, NULL);
 		link = TRUE;
-		result = html_engine_save_delims_and_vals (state, "<A HREF=\"", url, "\">", NULL);
+		result = html_engine_save_output_string (state, "<A HREF=\"%s\">", url);
 		g_free (url);
 		if (!result)
 			return FALSE;
 	}
 
 	url    = html_image_resolve_image_url (state->engine->widget, image->image_ptr->url);
-	result = html_engine_save_delims_and_vals (state, "<IMG SRC=\"", url, "\"", NULL);
+	result = html_engine_save_output_string (state, "<IMG SRC=\"%s\"", url);
 	g_free (url);
 	if (!result)
 		return FALSE;
@@ -648,7 +653,7 @@ save (HTMLObject *self,
 	}
 
 	if (image->alt) {
-		if (!html_engine_save_delims_and_vals (state, " ALT=\"", image->alt, "\"", NULL))
+		if (!html_engine_save_output_string (state, " ALT=\"%s\"", image->alt))
 			return FALSE;
 	}
 
@@ -956,7 +961,7 @@ html_image_set_border (HTMLImage *image, gint border)
 }
 
 void
-html_image_set_alt (HTMLImage *image, const gchar *alt)
+html_image_set_alt (HTMLImage *image, gchar *alt)
 {
 	g_free (image->alt);
 	image->alt = g_strdup (alt);
@@ -965,7 +970,7 @@ html_image_set_alt (HTMLImage *image, const gchar *alt)
 void
 html_image_set_map (HTMLImage *image, gchar *usemap, gboolean ismap)
 {
-	gchar *url = NULL;
+	char *url = NULL;
 
 	g_free (image->usemap);
 
@@ -1009,13 +1014,13 @@ html_image_set_size (HTMLImage *image, gint w, gint h, gboolean pw, gboolean ph)
 	}
 }
 
-static gchar *fallback_image_content_types[] = { (gchar *) "image/*", NULL};
+static char *fallback_image_content_types[] = {"image/*", NULL};
 
-static gchar **
+static char **
 html_image_factory_types (GtkHTMLStream *stream,
 			  gpointer user_data)
 {
-	static gchar **image_content_types = NULL;
+	static char**image_content_types = NULL;
 
 #if 0
 	/* this code should work in gtk+-2.2 but it is untested */
@@ -1029,7 +1034,7 @@ html_image_factory_types (GtkHTMLStream *stream,
 
 		for (cur = formats; cur; cur = cur->next) {
 			GdkPixbufFormat *format = cur->data;
-			gchar **mime;
+			char **mime;
 
 			mime = gdk_pixbuf_formats_get_mime_types ();
 			for (i = 0; mime && mime[i]; i++)
@@ -1039,7 +1044,7 @@ html_image_factory_types (GtkHTMLStream *stream,
 		g_slist_free (formats);
 
 		if (types) {
-			image_content_types = g_new0 (gchar *, g_slist_length (types) + 1);
+			image_content_types = g_new0 (char *, g_slist_length (types) + 1);
 
 			for (cur = types, i = 0; cur; cur = cur->next, i++) {
 				image_content_types[i] = cur->data;
@@ -1057,15 +1062,10 @@ html_image_factory_types (GtkHTMLStream *stream,
 }
 
 static void
-update_or_redraw (HTMLImagePointer *ip)
-{
+update_or_redraw (HTMLImagePointer *ip){
 	GSList *list;
 	gboolean update = FALSE;
 
-	/* do nothing when factory gone; maybe the message drawing has been cancelled */
-	if (!ip->factory)
-		return;
-
 	for (list = ip->interests; list; list = list->next) {
 		if (list->data == NULL)
 			update = TRUE;
@@ -1126,28 +1126,24 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 	g_object_unref (ip->loader);
 	ip->loader = NULL;
 
-	/* if no ip->factory is set, then the image loading has been cancelled meanwhile, probably. */
-	if (ip->factory) {
-		update_or_redraw (ip);
-		if (ip->factory->engine->opened_streams && ip->factory->engine->block_images)
-			html_engine_opened_streams_decrement (ip->factory->engine);
-		/* printf ("IMAGE(%p) opened streams: %d\n", ip->factory->engine, ip->factory->engine->opened_streams); */
-		if (ip->factory->engine->opened_streams == 0 && ip->factory->engine->block && ip->factory->engine->block_images)
-			html_engine_schedule_update (ip->factory->engine);
-	}
-
+	update_or_redraw (ip);
+	if (ip->factory->engine->opened_streams && ip->factory->engine->block_images)
+		html_engine_opened_streams_decrement (ip->factory->engine);
+	/* printf ("IMAGE(%p) opened streams: %d\n", ip->factory->engine, ip->factory->engine->opened_streams); */
+	if (ip->factory->engine->opened_streams == 0 && ip->factory->engine->block && ip->factory->engine->block_images)
+		html_engine_schedule_update (ip->factory->engine);
 	html_image_pointer_unref (ip);
 }
 
 static void
 html_image_factory_write_pixbuf (GtkHTMLStream *stream,
 				 const gchar *buffer,
-				 gsize size,
+				 size_t size,
 				 gpointer user_data)
 {
 	HTMLImagePointer *p = user_data;
 
-	/* FIXME !Check return value */
+	/* FIXME ! Check return value */
 	gdk_pixbuf_loader_write (p->loader, (const guchar *) buffer, size, NULL);
 }
 
@@ -1170,12 +1166,10 @@ html_image_pointer_queue_animation (HTMLImagePointer *ip)
 static gint
 html_image_pointer_update (HTMLImagePointer *ip)
 {
-	HTMLEngine *engine;
+	HTMLEngine *engine = ip->factory->engine;
 	GSList *cur;
 
 	g_return_val_if_fail (ip->factory != NULL, FALSE);
-
-	engine = ip->factory->engine;
 	ip->animation_timeout = 0;
 
 	DA (printf ("animation_timeout (%p)\n", ip);)
@@ -1263,7 +1257,7 @@ cleanup_images (gpointer key, gpointer value, gpointer free_everything)
 	HTMLImagePointer *ip = value;
 
 	/* free_everything means: NULL only clean, non-NULL free */
-	if (free_everything) {
+	if (free_everything){
 		if (ip->interests != NULL) {
 			g_slist_free (ip->interests);
 			ip->interests = NULL;
@@ -1271,7 +1265,7 @@ cleanup_images (gpointer key, gpointer value, gpointer free_everything)
 	}
 
 	/* clean only if this image is not used anymore */
-	if (!ip->interests) {
+	if (!ip->interests){
 		html_image_pointer_unref (ip);
 		ip->factory = NULL;
 		return TRUE;
@@ -1304,7 +1298,7 @@ html_image_factory_free (HTMLImageFactory *factory)
 #define STALL_INTERVAL 1000
 
 static HTMLImagePointer *
-html_image_pointer_new (const gchar *filename, HTMLImageFactory *factory)
+html_image_pointer_new (const char *filename, HTMLImageFactory *factory)
 {
 	HTMLImagePointer *retval;
 
@@ -1408,7 +1402,7 @@ html_image_pointer_unref (HTMLImagePointer *ip)
 static GtkHTMLStream *
 html_image_pointer_load (HTMLImagePointer *ip)
 {
-	if (!ip->factory || ip->factory->engine->stopped)
+	if (ip->factory->engine->stopped)
 		return NULL;
 
 	html_image_pointer_ref (ip);
@@ -1423,7 +1417,7 @@ html_image_pointer_load (HTMLImagePointer *ip)
 }
 
 HTMLImagePointer *
-html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const gchar *url, gboolean reload)
+html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char *url, gboolean reload)
 {
 	HTMLImagePointer *ip;
 	GtkHTMLStream *stream = NULL;
@@ -1577,6 +1571,7 @@ deactivate_anim (gpointer key, gpointer value, gpointer user_data)
 	GSList *cur = ip->interests;
 	HTMLImage *image;
 
+
 	while (cur) {
 		if (cur->data) {
 			image = (HTMLImage *) cur->data;
